# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:24
import hashlib
import json
from json import JSONEncoder

from six import iteritems

from ..compat import utf8encode


def md5sum(text):
  """
  获取文本的MD5值
  :param text: 文本内容
  :return: md5摘要值
  """
  _ctx = hashlib.md5()
  _ctx.update(text.encode('utf-8'))
  return _ctx.hexdigest()


def parse_json(json_str):
  """
  将json字符串解析为Python数据
  :param json_str: json字符串
  :return: python对象(dict)
  """
  return json.loads(json_str)


from_json = parse_json


def as_json(subject, **kwargs):
  """
  将subject转换为json字符串
  :param subject: 待转换对象
  :type subject: object
  :param kwargs: 其余参数
  :return: 字符串
  """
  if 'ensure_ascii' not in kwargs:
    kwargs['ensure_ascii'] = False
  utf8 = 'utf8' in kwargs
  if utf8:
    kwargs.pop('utf8')
  data = json.dumps(_ensure_type(subject), **kwargs)
  return utf8encode(data) if utf8 else data


def _ensure_type(subject):
  if not (subject is None
          or isinstance(subject, bool)
          or isinstance(subject, tuple)
          or isinstance(subject, list)
          or isinstance(subject, set)
          or isinstance(subject, str)
          or isinstance(subject, int)
          or isinstance(subject, float)
          or isinstance(subject, dict)
          or isinstance(subject, JSONEncoder)):
    return str(subject)

  if isinstance(subject, tuple):
    return tuple(_ensure_type(x) for x in subject)
  if isinstance(subject, list):
    return list(_ensure_type(x) for x in subject)
  if isinstance(subject, set):
    return list(_ensure_type(x) for x in subject)
  if isinstance(subject, dict):
    return {
      k: _ensure_type(v) for k, v in iteritems(subject)
    }
  return subject


to_json = as_json



from .compat import C_MAXINT
from .compat import C_StandardError
from .compat import compat_warning
from .compat import cse_message
from .compat import utf8decode
from .compat import utf8encode
from .filtering import filter_dict_object
from .filtering import filter_module_methods
from .functional import async_call
from .functional import deprecated
from .functional import none_context
from .invoking import executable_exists
from .invoking import python2_invoke
from .invoking import python3_invoke
from .invoking import python_invoke
from .invoking import shell_invoke
from .utility import it_print
from .utility import it_prints



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/14 10:53
import sys

import six

python2 = six.PY2
python3 = six.PY3

if python2:
  C_StandardError = Exception
  C_MAXINT = sys.maxint

  from six.moves import reload_module

  reload_module(sys)
  sys.setdefaultencoding('utf-8')

if python3:
  C_StandardError = BaseException
  C_MAXINT = sys.maxsize


def cse_message(e):
  """
  获取异常消息
  :param e: 异常实例
  :return: 异常消息
  """
  assert isinstance(e, C_StandardError)

  if python2:
    result = e.message
    if result == '':
      result = str(e)
    return result

  if python3:
    if len(e.args) >= 1:
      return e.args[0]
    return ''


def utf8decode(text):
  """
  将text解码为unicode
  :param text: 待解码字符
  :return: 解码后的内容
  """
  return text.decode('utf-8')


def utf8encode(text):
  """
  将text编码为UTF8
  :param text: 待编码内容
  :return: UTF8编码
  """
  return text.encode('utf-8')


TYPE_NAME = 'type' if python2 else 'class'
TYPE_FUNCTION = "<{} 'function'>".format(TYPE_NAME)
TYPE_CLASS = "<{} 'type'>".format(TYPE_NAME)


def compat_warning(new_module, *functions):
  """
  用于无警告移动函数
  :param new_module: 新模块
  :param functions: 移动的函数名
  """
  import warnings

  class_count, func_count = 0, 0
  moved_classes, moved_functions = [], []
  for fn in functions:
    type_name = str(type(fn))
    if type_name == TYPE_CLASS:
      class_count += 1
      moved_classes.append(fn.__name__)
    elif type_name == TYPE_FUNCTION:
      func_count += 1
      moved_functions.append(fn.__name__)

  kwargs = {
    'module_name': new_module.__name__,
    'class_count': class_count,
    'func_count': func_count,
    'moved_classes': ','.join(moved_classes),
    'moved_functions': ','.join(moved_functions),

  }

  fmts = ['\n', 'Compatibility warning!\n']
  _fmts = ['  ']
  if class_count > 0:
    _fmts.append('{class_count} classes and ')
  if func_count > 0:
    _fmts.append('{func_count} functions ')
  _fmts.append('have been moved to "{module_name}".\n')
  fmts.append(''.join(_fmts))

  if class_count > 0:
    fmts.append('  classes: "{moved_classes}".\n')
  if func_count > 0:
    fmts.append('  functions: "{moved_functions}".\n')
  fmts.append('  please upgrade to the newest version.')

  warnings.warn(''.join(fmts).format(**kwargs))



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/9 14:36
from contextlib import contextmanager
from threading import Thread


def deprecated(new_fn=None, version='0.7'):
  """
  标记为弃用 decorator
  :param new_fn: 新的替代函数
  :param version: 被移除版本
  :return 包装函数
  """
  if new_fn is not None:
    assert callable(new_fn)

  def function_wrapper(fn):
    assert callable(fn)

    def wrapper(*args, **kwargs):
      if version is None:
        future = 'the future'
      else:
        future = 'version {}'.format(str(version))

      params = {
        'fn_module': fn.__module__,
        'fn_name': fn.__name__,
        'future': future
      }
      fmts = list(['\n', 'Deprecated warning!\n'])
      fmts.append((
        '  some of your code are using deprecated function: \n'
      ))
      fmts.append((
        '    [{fn_name}] from [{fn_module}].\n'
      ))
      fmts.append((
        '  the deprecated will be removed in {future}.\n'
      ))

      if new_fn is not None:
        params['new_fn_module'] = new_fn.__module__
        params['new_fn_name'] = new_fn.__name__
        fmts.append((
          '  use [{new_fn_name}] from [{new_fn_module}] instead.'
          ''))

      import warnings
      message = ''.join(fmts)
      warnings.warn(message.format(**params))

      return fn(*args, **kwargs)

    return wrapper

  return function_wrapper


def async_call(daemon=False, name=None):
  """
  异步调用 decorator
  :param daemon: 是否为守护进程
  :param name: 线程名称
  :return 包装函数
  """

  def function_wrapper(fn):
    """
    函数修饰器
    :param fn: 被调用函数
    """
    assert callable(fn)

    def wrapper(*args, **kwargs):
      t = Thread(target=fn, args=args, kwargs=kwargs)
      t.setDaemon(daemon)
      if name is not None:
        t.setName(name)
      t.start()

    return wrapper

  return function_wrapper


@contextmanager
def none_context():
  yield



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:32
import urllib

import six
from six.moves import xrange

from .collection import Iterable
from .stringext import to_json
from ..compat import C_StandardError


class TimeoutError(Exception):
  """
  # 超时标记接口
  """
  pass


def build_url(host, port, context, uri, schema='http'):
  """
  构建url
  :param host: 主机
  :param port: 端口
  :param context: 上下文
  :param uri: URI路径
  :param schema: 请求模式
  :return: 完整的url
  """
  assert schema == 'http' or schema == 'https'
  return '{}://{}:{}{}{}'.format(schema, host, port, context, uri)


def request(request_url, headers=None, data=None, as_json=True, timeout=None):
  """
  发送HTTP请求
  :param request_url: 请求url
  :param headers: 头部
  :param data: 请求数据
  :param as_json: 是否以json格式发送数据
  :param timeout: 超时时间
  :return: 响应结果或超时异常
  """
  kwargs = dict()
  if timeout is not None:
    kwargs['timeout'] = timeout

  _headers = dict()
  if headers is not None:
    assert isinstance(headers, dict)
    _headers.update(headers)

  result = None
  if six.PY2:
    from urllib2 import Request
    from urllib2 import build_opener
    from urllib2 import HTTPCookieProcessor
    from urllib2 import URLError

    req = Request(request_url, headers=_headers)
    opener = build_opener(HTTPCookieProcessor())
    if data is not None:
      if as_json:
        data = to_json(data, utf8=True)
      else:
        data = urllib.urlencode(data)
    try:
      response = opener.open(req, data, **kwargs)
      result = response.read()
    except URLError as e:
      # e.reason: 输出如，[Errno 61] Connection refused，但Mac和Unix的Errno不一样
      if str(e.reason).find('Connection refused') != -1:
        raise TimeoutError
    except C_StandardError as e:
      if e.message == 'timed out':
        raise TimeoutError
  else:
    from urllib3 import PoolManager
    from urllib3.exceptions import MaxRetryError
    from urllib3.exceptions import NewConnectionError
    from urllib3.exceptions import ReadTimeoutError
    from urllib3.exceptions import RequestError

    http = PoolManager()
    if data is not None:
      if as_json:
        kwargs['body'] = to_json(data, utf8=True)
      else:
        kwargs['fields'] = data
    try:
      response = http.request(
        'POST', request_url, headers=_headers, **kwargs
      )
      result = response.data
    except RequestError as e:
      if isinstance(e, MaxRetryError) \
        and isinstance(e.reason, NewConnectionError):
        raise TimeoutError
      if isinstance(e, ReadTimeoutError):
        raise TimeoutError
  return result


def fetch_available_port(host='127.0.0.1', used_ports=None, port_range=None):
  """
  获取可用的端口号
  :param host: 主机
  :param used_ports: 端口
  :param port_range: 端口范围
  :return: 端口号
  """
  import socket

  if used_ports is None:
    used_ports = []

  if port_range is None:
    port_range = xrange(50000, 63000)
  else:
    if not isinstance(port_range, Iterable):
      s = 'port_range should be of Iterable type, but got {}'
      raise ValueError(s.format(type(port_range)))

  for port in port_range:
    if port in used_ports:
      continue

    try:
      sk = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
      sk.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
      sk.bind((host, port))
      sk.close()
      return port
    except C_StandardError as e:
      if str(e.args[1]) == 'Address already in use':
        continue
  raise C_StandardError('Can\'t get any available port')



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 09:30
from __future__ import division
from __future__ import print_function

import functools
import sys

from .stringext import as_json
from ..compat import python3


def _add_indent(lines, space_cnt):
  """
  add blank before message

  :param lines: lines to be operated
  :type lines: Iterable of str

  :param space_cnt: space number
  :type space_cnt: int

  :return: lines with each started with space_cnt blanks
  """
  s = lines.split('\n')
  # don't do anything for single-line stuff
  if len(s) == 1:
    return lines
  first = s.pop(0)
  s = [(space_cnt * ' ') + line for line in s]
  s = '\n'.join(s)
  s = first + '\n' + s
  return s


"""
Whether enable message out or not
"""
_enable = True


def it_print(message='', indent=None, device=1, newline=True, json_fmt=False):
  """
  in time print: print one line to console immediately

  :param message: the message to be printed
  :type message: object

  :param indent: number of blank to be indented, default 0
  :type indent: int

  :param device: stdout -> 1, stderr -> 2, default 1
  :type device: int

  :param newline: whether to append a new line, default True
  :type newline: bool

  :param json_fmt: whether to print as json
  :type json_fmt: bool
  """
  global _enable
  if not _enable or device is None:
    return

  if message is None:
    message = str(None)
  else:
    if not json_fmt:
      if indent is None:
        indent = 0
      message = ' ' * indent + str(message)
    else:
      message = as_json(message, indent=indent)

  if device == 2:
    device = sys.stderr
    message = '\033[0;31m%s\033[0m' % message
  elif device == 1:
    device = sys.stdout

  # a file-like object
  assert hasattr(device, 'write')

  kwargs = dict(file=device,
                end='\n' if newline else '')
  if python3:
    kwargs['flush'] = True
  print(message, **kwargs)


def it_prints(message='', indent=None, indent_inner=2, device=1, newline=True):
  """
  in time print multiple lines: first indent with indent blanks, then every line is indented with indent_inner blanks
  """
  if message is not None:
    message = _add_indent(message, indent_inner)
  it_print(message, indent=indent, device=device, newline=newline)


def _set_enabled(enable):
  global _enable
  _enable = enable


def is_print_enable():
  """
  get the current printable status
  """
  return _enable


def set_print_enable():
  _set_enabled(True)


def set_print_disable():
  _set_enabled(False)


class disable_print(object):
  def __init__(self):
    self.prev = is_print_enable()

  def __enter__(self):
    set_print_disable()

  def __exit__(self, *args):
    _set_enabled(self.prev)
    return False

  def __call__(self, func):
    @functools.wraps(func)
    def decorate_disable_print(*args, **kwargs):
      with self:
        return func(*args, **kwargs)

    return decorate_disable_print


class enable_print(object):
  def __init__(self):
    self.prev = is_print_enable()

  def __enter__(self):
    set_print_enable()

  def __exit__(self, *args):
    _set_enabled(self.prev)
    return False

  def __call__(self, func):
    @functools.wraps(func)
    def decorate_disable_print(*args, **kwargs):
      with self:
        return func(*args, **kwargs)

    return decorate_disable_print



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:25
from __future__ import print_function

from .printext import it_print
from .stringext import parse_json
from .stringext import to_json
from .datetime import Timer
from .webext import request
from ..functional import deprecated


def print_indent(message):
  """
  缩进打印
  :param message: 待打印消息
  """
  it_print(message, indent=2)


def print_prefix(subject, msg=None):
  """
  以某个消息为前缀打印
  :param subject: 打印内容
  :param msg: 消息前缀
  """
  if msg is not None:
    subject = msg + ' ' + subject
  it_print(subject)


def print_dict(data, msg=None):
  """
  打印字典
  :param data: 数据
  :param msg: 消息提要
  """
  if msg is not None:
    it_print(msg)
  for key in data:
    value = data[key]
    print_indent('{} => {}'.format(key, value))


def json_print(data):
  """
  以json形式打印
  :param data: 待打印数据
  """
  it_print(to_json(data, indent=2, sort_keys=True, ensure_ascii=False))


def print_json(json_string):
  """
  打印json字符串
  :param json_string: json字符串
  :return:
  """
  json_print(parse_json(json_string))


class HttpTest(object):
  """
  Http API测试工具
  """
  debug = True

  def __init__(self, url=None):
    self.url = url

  def set_url(self, host, context, port=80, https=False):
    schema = 'https' if https else 'http'
    self.url = '{}://{}:{}{}'.format(schema, host, port, context)

  def request_url(self, url, data=None, as_json=True, timeout=None):
    # 请求地址
    request_url = url
    if self.debug:
      print_prefix(request_url, "url:")

    # 请求数据
    if data is None:
      data = {}

    if self.debug:
      # 起始时间
      timer = Timer()
      timer.display_start("request at: ")

      # 打印请求数据
      it_print("request:")
      if len(data) != 0:
        json_print(data)

    # 发起请求
    res = request(request_url, data, as_json, timeout)

    if self.debug:
      timer.finish()
      # 打印结果
      it_print("response:")
      if res is not None:
        print_json(res)
      else:
        it_print()

      # 结束时间
      timer.display_finish("finished at: ")
      it_print("time cost: {} s".format(timer.cost()))

      it_print()
    return res

  def request(self, uri, data=None, as_json=True, timeout=None):
    return self.request_url('{}{}'.format(self.url, uri), data, as_json, timeout)



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:01
from typing import Iterable


def unique_list(items):
  """
  获取无重复的列表
  :param items: 可迭代对象
  :return: 无重复的list对象
  """
  if not isinstance(items, Iterable):
    return items
  return list(set(items))



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:30
from six.moves import xrange

from .collection import Iterable


def contain_keys(data, *keys):
  """
  检测字典data中是否包含指定的所有键
  :param data: 数据
  :param keys: 键集
  :return: 若data包含keys中所有键则返回True, 否则返回False
  """
  if len(keys) == 1:
    keys = keys[0]

  if not isinstance(keys, Iterable):
    return False

  for key in keys:
    if key not in data:
      return False
  return True


def in_range(num, range_from, range_to):
  """
  检测数字是否在范围[range_from, range_to]内
  :param num: 待检测数值
  :param range_from: 起始范围
  :param range_to: 结束范围
  :return: 若num在范围内返回True, 否则返回False
  """
  return num in xrange(range_from, range_to + 1)



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:22
import signal
from os import kill

import psutil

from . import exit_print
from .filesystem import file_exists
from .filesystem import load_file_contents
from .filesystem import remove_file
from .printext import it_print


def kill_process(pid_path, signum=None, remove=True):
  """
  向pid_path文件内容中的PID发送信号
  :param pid_path: pid文件路径
  :param signum: 信号值
  :param remove: 是否移除pid文件
  """
  if not file_exists(pid_path):
    return None

  if signum is None:
    signum = signal.SIGKILL

  # 读取PID文件
  pid = int(load_file_contents(pid_path)[0].strip())

  # 向该进程发送信号
  try:
    kill(pid, signum)
  except OSError as e:
    if len(e.args) != 2 or e.args[1] != 'No such process':
      return None

  # 移除PID文件
  if remove:
    remove_file(pid_path)

  return True


def query_process(pid_path):
  """
  :param pid_path PID文件路径
  """
  if not file_exists(pid_path):
    exit_print('{} does not exist, please check it manually!'.format(pid_path))

  pid = int(load_file_contents(pid_path, pieces=False))
  if psutil.pid_exists(pid):
    it_print("Process is running with PID={} !".format(pid))
  else:
    it_print("Process is not running with PID={} !".format(pid))



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018年8月23日 14:26:49
import asyncio
import json
import time
from concurrent.futures import ThreadPoolExecutor
from functools import partial

from tornado.concurrent import run_on_executor
from tornado.httpclient import AsyncHTTPClient
from tornado.httpclient import HTTPError
from tornado.httpclient import HTTPRequest
from tornado.httpclient import HTTPResponse
from tornado.web import RequestHandler

from ..compat import C_StandardError
from ..compat import utf8decode
from ..utility.httpclient import json_print
from ..utility.printext import it_print
from ..utility.stringext import to_json
from ..utility.datetime import current_datetime


class WebApplicationHandler(RequestHandler):
  # 全局信息
  global_map = dict()

  # 错误信息
  error_mapper = dict()

  # 正常响应
  none = 0
  error_mapper[none] = 'none'

  # 无效请求(参数错误)
  invalid_request = 1001
  error_mapper[invalid_request] = 'request param is invalid'

  # 数据未找到
  not_found = 4004
  error_mapper[not_found] = 'not found'

  # 操作不支持
  not_support = 5001
  error_mapper[not_support] = 'not support'

  # 本系统错误
  system_error = 5002
  error_mapper[system_error] = 'system error'

  # 内部服务器错误
  internal_server = 5003
  error_mapper[internal_server] = 'internal server'

  # 请求超时
  request_timeout = 5005
  error_mapper[request_timeout] = 'request timeout'

  # 调试模式
  debug = False

  # 开发模式
  devel = True

  # 作为后台进程运行
  daemon = True

  default_key = 'default'

  data_key = 'data'

  r_data_key = 'r_data'

  q_data_key = 'q_data'

  # 线程池执行器
  executor = None

  @staticmethod
  def setup_config(**kwargs):
    cls = WebApplicationHandler
    cls.debug = kwargs.pop('debug', cls.debug)
    cls.devel = kwargs.pop('devel', cls.devel)
    cls.daemon = kwargs.pop('daemon', cls.daemon)
    cls.executor = kwargs.pop('executor', ThreadPoolExecutor(1))

  @run_on_executor
  def apply_async(self, invoker):
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    invoker()

  @staticmethod
  def persist(**kwargs):
    pass

  @staticmethod
  def restore(**kwargs):
    cls = WebApplicationHandler
    cls.global_map.update(kwargs)

  # 加载为json数据
  def load_request_data(self):
    try:
      if self.debug:
        it_print(self.request.body)
      body = utf8decode(self.request.body)
      params = json.loads(body) if body != '' else dict()
      # 加入私有属性作为数据，否则params为空，被if not判断为真
      params['_uri'] = self.request.uri
      if self.debug:
        params['request_time'] = current_datetime()
        self.pretty_it_print(params)
    except ValueError:
      return False
    return params

  # 加载query数据
  def load_query_data(self):
    try:
      if self.debug:
        it_print(self.request.query)
      params = {
        k: [utf8decode(x) for x in v]
        for k, v in self.request.query_arguments.items()
      }
    except ValueError:
      return False
    return params

  # 异步转发请求
  @staticmethod
  def async_forward(url, data=None, callback=None,
                    method='POST', timeout=3600):
    if callback is None:
      def empty_fn(_):
        pass

      callback = empty_fn

    _data = dict()
    if data is not None:
      assert isinstance(data, dict)
      _data.update(data)

    json_data = to_json(_data)
    client = AsyncHTTPClient()
    future = client.fetch(
      url, body=json_data, method=method, request_timeout=timeout
    )

    def handle_future(_future):
      exc = future.exception()
      if isinstance(exc, HTTPError) and exc.response is not None:
        response = exc.response
      elif exc is not None:
        request = HTTPRequest(url)
        response = HTTPResponse(
          request, 599, error=exc,
          request_time=time.time() - request.start_time)
      else:
        response = _future.result()
      client.io_loop.add_callback(callback, response)

    future.add_done_callback(handle_future)

  # 同步转发请求
  async def forward(self, url, data=None, callback=None,
                    method='POST', timeout=3600):
    if callback is None:
      callback = partial(self.response, inplace=True)

    _data = dict()
    if data is not None:
      assert isinstance(data, dict)
      _data.update(data)

    json_data = to_json(_data)
    client = AsyncHTTPClient()
    response = await client.fetch(
      url, body=json_data, method=method,
      request_timeout=timeout, connect_timeout=timeout
    )
    return callback(response)

  def success_response(self, data=None, inplace=False):
    if inplace:
      self.error_response(data.pop('errno'), data.pop('error'), data)
    else:
      self.error_response(self.none, self.error_mapper[self.none], data)

  def error_response(self, errno=invalid_request, error=None, data=None):
    res = dict()
    if data is not None:
      assert isinstance(data, dict)
      res.update(data)

    res['errno'] = errno
    if error is None:
      error = self.error_mapper[errno]
    res['error'] = error

    self.__json_response(res)

  def text_response(self, text):
    self.__output_response(text)

  def json_response(self, data):
    self.__json_response(data)

  def __json_response(self, data):
    if self.debug:
      data['response_time'] = current_datetime()
      self.pretty_it_print(data)
    self.__output_response(to_json(data, sort_keys=True))

  def __output_response(self, data):
    self.write(data)
    self.finish()

  # 默认响应处理器
  def response(self, response, inplace=False):
    try:
      if self.debug:
        it_print(response.body)
      result = json.loads(response.body)
    except ValueError:
      return self.error_response(errno=self.system_error)
    return self.success_response(result, inplace=inplace)

  def data_received(self, chunk):
    pass

  @staticmethod
  def pretty_it_print(data):
    json_print(data)

  @staticmethod
  def exists(key):
    return key in WebApplicationHandler.global_map

  @staticmethod
  def store(key, value):
    self = WebApplicationHandler
    self.global_map[key] = value

  @staticmethod
  def release(key):
    self = WebApplicationHandler
    if self.exists(key):
      self.global_map.pop(key)

  @staticmethod
  def retrieve(key, default=None):
    self = WebApplicationHandler
    if self.exists(key):
      return self.global_map[key]
    return default

  def p(self, key, data=None, **kwargs):
    if key in data:
      return data[key]

    if self.default_key in kwargs:
      return kwargs[self.default_key]

    raise C_StandardError(
      'request parameter [{}] does not present'.format(key)
    )

  def q(self, key, data=None, multiple=False, type_fn=str, **kwargs):
    if data is None:
      data = self.load_query_data()

    if key not in data:
      if self.default_key in kwargs:
        return kwargs[self.default_key]

      return C_StandardError(
        'query parameter [{}] does not present'.format(key)
      )

    values = data[key]
    if multiple:
      return [type_fn(x) for x in values]
    return type_fn(values[0])

  def v(self, key, **kwargs):
    if self.data_key in kwargs:
      data = kwargs[self.data_key]
      if key in data:
        return data[key]

    if self.q_data_key in kwargs:
      data = kwargs[self.q_data_key]
    else:
      data = self.load_query_data()
    if key in data:
      values = data[key]
      multiple = kwargs.pop('multiple', False)
      type_fn = kwargs.pop('type_fn', str)
      if multiple:
        return [type_fn(x) for x in values]
      return type_fn(values[0])

    if self.r_data_key in kwargs:
      data = kwargs[self.r_data_key]
    else:
      data = self.load_request_data()
    if key in data:
      return data[key]

    if self.default_key in kwargs:
      return kwargs[self.default_key]

    raise C_StandardError(
      'parameter [{}] does not present'.format(key)
    )



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:07
import codecs
import hashlib
import io
import os
import shutil
import subprocess
import tempfile
from functools import partial

from decorator import contextmanager

from .collection import Iterable
from .stringext import from_json
from .stringext import to_json
from ..compat import utf8encode


def abspath(file_obj):
  """
  获取文件绝对路径, 一般在调用文件传__file__对象
  :param file_obj: 文件对象
  :return: 文件的绝对路径
  """
  return os.path.abspath(file_obj)


def absdir(file_obj):
  """
  获取文件所在目录绝对路径, 一般在调用文件传__file__对象
  :param file_obj: 文件对象
  :return: 文件的所在的目录的绝对路径
  """
  _abspath = abspath(file_obj)
  return os.path.dirname(_abspath)


def dirname(path):
  """
  获取路径父路径
  :param path: 路径
  :return: 父路径
  """
  return os.path.dirname(path)


def file_exists(path):
  """
  判断文件是否存在
  :param path: 文件路径
  :return: 文件存在返回True, 否则返回False
  """
  return os.path.islink(path) or os.path.exists(path)


def file_size(path):
  """
  获取文件大小
  :param path: 文件路径
  :return: 文件大小
  """
  return os.path.getsize(path)


def file_lines(path):
  """
  获取文件行数, 通过Shell命令
  :param path: 文件路径
  :return: 文件行数
  """
  if not file_exists(path):
    return -1

  return sum(1 for _ in open(path))


def remove_file(path, ignore_errors=True):
  """
  移除文件(文件及目录)
  :param path: 文件路径
  :param ignore_errors: 是否忽略错误
  :raise 若路径不存在, 则无操作; 若路径既非文件且又非目录则产生ValueError
  """
  if not file_exists(path):
    return

  if os.path.isfile(path) or os.path.islink(path):
    os.remove(path)
  elif os.path.isdir(path):
    shutil.rmtree(path, ignore_errors=ignore_errors)
  else:
    raise ValueError('path should be either a file or directory')


def create_if_not_exists(path):
  """
  如果对应路径不存在的话，则进行创建
  :param path: 文件路径
  :return 若文件已存在且非目录, 抛出ValueError异常
  """
  if os.path.exists(path) and not os.path.isdir(path):
    raise ValueError('path {} exists but is not a directory')

  if not os.path.exists(path):
    os.makedirs(path, exist_ok=True)


def concat_path(base_path, sub_path=None, utf8=False):
  """
  拼接路径
  :param base_path: 基本路径
  :param sub_path: 子路径
  :param utf8: 控制是否使用utf8编码(如传输C库)
  :return: 拼接后的路径
  """
  assert base_path is not None
  if sub_path is None:
    path = base_path
  else:
    path = os.path.join(base_path, sub_path)
  return utf8encode(path) if utf8 else path


"""
  拼接路径函数别名
"""
cp = concat_path

"""
  拼接作为C库的路径参数
"""
clp = partial(concat_path, utf8=True)


def is_abspath(file_path):
  """
  判断是否为绝对路径
  :param file_path: 文件路径
  :return: 若为绝对路径返回True, 否则返回False
  """
  cond = file_path is not None and isinstance(file_path, str)
  return cond and file_path.startswith('/')


def basename(file_path):
  return os.path.basename(file_path)


def refine_path(base_path, holder, key):
  """
  修正路径(将holder中key对应的值增加base_path)
  :param base_path: 基本路径
  :param holder: 路径容器
  :param key: 路径键或索引
  """
  if isinstance(holder, dict):
    assert key in holder
  elif isinstance(holder, list):
    assert isinstance(key, int) and 0 <= key < len(holder)
  else:
    s = 'holder should be either dict or list, but got {}'
    raise TypeError(s.format(type(holder)))

  if not is_abspath(holder[key]):
    holder[key] = cp(base_path, holder[key])


def file_md5sum(file_path):
  """
  计算文件的MD5值
  :param file_path: 文件路径
  :return: 文件MD5
  """
  if not os.path.isfile(file_path):
    return
  ctx = hashlib.md5()
  with open(file_path, 'rb') as fp:
    while True:
      b = fp.read(8096)
      if not b:
        break
      ctx.update(b)
  return ctx.hexdigest()


def file_append(path_append_to, path_append_from):
  """
  将path_append_from的文件内容追加文件到path_append_to中
  :param path_append_to: 待追加内容的文件
  :param path_append_from: 待追加内容所在文件
  :return: 追加结果
  """
  kwargs = {
    'path_append_from': path_append_from,
    'path_append_to': path_append_to
  }
  if not file_exists(path_append_to):
    cmd_create_fmt = 'cp {path_append_from} {path_append_to}'
    cmd_create_str = cmd_create_fmt.format(**kwargs)
    try:
      subprocess.check_call(cmd_create_str, shell=True)
    except subprocess.CalledProcessError:
      return False
  else:
    cmd_append_fmt = 'cat {path_append_from} >> {path_append_to}'
    cmd_append_str = cmd_append_fmt.format(**kwargs)
    try:
      subprocess.check_call(cmd_append_str, shell=True)
    except subprocess.CalledProcessError:
      return False
  return True


def load_file_contents(path, pieces=True, strip=True, glue='', **kwargs):
  """
  读取文件内容
  :param path: 文件路径
  :param pieces: 是否按行返回
  :param strip: 是否对每行进行strip操作
  :param glue: 将pieces连接在一起的符号
  :return: 若文件不存在返回None, 若可正确读取则返回按行分割的内容列表
  """
  if not file_exists(path):
    return None

  if 'encoding' not in kwargs:
    kwargs['encoding'] = 'UTF-8'

  with codecs.open(path, 'r', **kwargs) as fp:
    lines = fp.readlines()
    if strip:
      lines = [x.strip() for x in lines]

    if pieces:
      return lines

    return glue.join([x for x in lines])


def load_json_contents(path):
  return from_json(load_file_contents(path, pieces=False, strip=True))


def write_line(wfp, line):
  """
  向文件中写入一行
  :param wfp: 文件写对象
  :param line: 行内容
  """
  if wfp:
    wfp.write(line.strip())
    wfp.write('\n')


def write_pid(path, **kwargs):
  """
  将pid写入到路径所在文件
  :param path: 路径
  """
  if 'encoding' not in kwargs:
    kwargs['encoding'] = 'UTF-8'
  with codecs.open(path, 'w', **kwargs) as wfp:
    wfp.write(str(os.getpid()))


def write_file_contents(path, contents, newline=False, **kwargs):
  """
  写入内容至文件
  :param path: 文件路径
  :param contents: 待写入内容
  :param newline: 是否追加新行
  """
  if 'encoding' not in kwargs:
    kwargs['encoding'] = 'UTF-8'
  with codecs.open(path, 'w', **kwargs) as wfp:
    wfp.write(contents)
    if newline:
      wfp.write('\n')


def write_json_contents(path, data, newline=False):
  """
  写入JSON内容
  :param path: 文件路径
  :param data: 待写入数据
  :param newline: 是否追加新行
  """
  json_str = to_json(data)
  write_file_contents(path, json_str, newline=newline)


def write_iterable_contents(path, iterable_obj, obj2line_func=lambda x: x):
  """
  将可迭代的数据按行的形式写入文件
  :param path: 文件路径
  :param iterable_obj: 可迭代对象
  :param obj2line_func: 将对象映射为行的函数
  """
  with open(path, 'w') as wfp:
    for obj in iterable_obj:
      write_line(wfp, obj2line_func(obj))


@contextmanager
def work_dir(path=None):
  cwd = os.getcwd()
  try:
    if path is not None:
      os.chdir(path)
      yield
  finally:
    os.chdir(cwd)


@contextmanager
def mkdtemp():
  """
  创建临时路径, 并在退出域时删除该路径
  :return 临时路径
  """
  path = tempfile.mkdtemp()
  create_if_not_exists(path)
  yield path
  shutil.rmtree(path)


@contextmanager
def mkstemp():
  """
  创建临时文件, 并在退出域时删除该路径
  :return 临时路径
  """
  path = tempfile.mkstemp()
  yield path[1]
  remove_file(path[1])


@contextmanager
def open_files(*paths, **kwargs):
  """
  批量打开文件
  :param paths: 文件路径
  :param kwargs: 关键字参数(包含打开文件句柄的函数open_fn及相应的函数调用关键字参数)
  :return: 文件句柄
  """
  handles = []
  if not isinstance(paths, Iterable):
    paths = [paths]

  open_fn = kwargs.pop('open_fn', io.open)
  if not callable(open_fn):
    s = 'open_fn should be callable type, but got {}'
    raise TypeError(s.format(type(open_fn)))

  for path in paths:
    handles.append(open_fn(path, **kwargs))

  yield handles

  for handle in handles:
    handle.close()



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:03
from .printext import it_print
from ..compat import C_StandardError


class InternalError(C_StandardError):

  def __init__(self, *args, **kwargs):
    super(InternalError, self).__init__(*args, **kwargs)


def raise_print(*args, **kwargs):
  """
  print message and raise InternalError
  :param args: see it_print
  :param kwargs: it_print
  """
  _do_print(*args, **kwargs)
  message = kwargs.pop('message', 'Unknown')
  raise InternalError(message)


def exit_print(*args, **kwargs):
  """
  print message and exit
  :param args: see it_print
  :param kwargs: see it_print
  """
  _do_print(*args, **kwargs)
  errno = kwargs.pop('errno', 0)
  exit(int(errno))


def _do_print(*args, **kwargs):
  if 'device' not in kwargs:
    kwargs['device'] = 2

  print_fn = kwargs.pop('print_fn', it_print)
  print_fn(*args, **kwargs)



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/11 12:42
from six import exec_


def filter_dict_object(data, *keys, **kwargs):
  """
  获取字典中想要/不要的键, 取决于kwargs参数
  :param data: 字典对象
  :param keys: 需要保留的键
  :param kwargs: 关键字参数
  :return 过滤掉的字典内容
  """
  if not isinstance(data, dict):
    raise TypeError('expected dict type, but got {}'.format(type(data)))
  result = dict()
  keep = False
  if 'keep' in kwargs and kwargs.pop('keep'):
    keep = True
  all_keys = list(data.keys())
  for key in all_keys:
    if keep:
      if key not in keys:
        result[key] = data.pop(key)
    else:
      if key in keys:
        result[key] = data.pop(key)
  return result


def filter_module_methods(module_object, filter_fn=None):
  """
  列出模块方法
  :param module_object: 模块对象
  :param filter_fn: 过滤函数 lambda 方法名: bool
  :return: 满足条件的方法列表
  """

  def _filter_fn_default(x):
    return x.strip() != '' and x[0].islower() and not (x.startswith('_') or x.startswith('__'))

  def _filter_fn_method(x, _):
    exec_('import sys')
    return eval('callable(sys.modules[_.__name__].{})'.format(x))

  if filter_fn is None:
    filter_fn = _filter_fn_default

  def fn(x):
    return filter_fn(x) and _filter_fn_method(x, module_object)

  return list(filter(fn, dir(module_object)))



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/19 11:28
from __future__ import division

import datetime
import time

from .printext import it_print
from ..functional import deprecated


def current_timestamp():
  """
  获取当前时间戳
  :return: 时间戳
  """
  return time.time()


def current_datetime_r():
  """
  获取当前datetime
  :return: 时间戳
  """
  return datetime.datetime.now()


def current_datetime(timestamp=None):
  """
  获取当前日期
  :param timestamp: 时间戳
  :return: 日期时间格式字符串 形如 2018-11-19 10:20:35
  """
  return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(timestamp))


def current_datetime_str(timestamp=None):
  """
  获取当前时间戳对应的日期时间字符串
  :param timestamp: 时间戳
  :return: 日期时间字符串 形如 20181119102035
  """
  return time.strftime("%Y%m%d%H%M%S", time.localtime(timestamp))


def current_datetime_str_s(timestamp=None):
  """
  获取当前时间戳对应的以T连接的日期、时间字符串
  :param timestamp: 时间戳
  :return: 日期时间字符串 形如 20181119T102035
  """
  return time.strftime("%Y%m%dT%H%M%S", time.localtime(timestamp))


def current_datetime_str_d(timestamp=None):
  """
  获取当前时间戳对应的以-连接的日期、时间字符串, 日期时间内容通过.连接
  :param timestamp: 时间戳
  :return: 日期时间字符串 形如 2018.11.19-10.20.35
  """
  return time.strftime("%Y.%m.%d-%H.%M.%S", time.localtime(timestamp))


def current_datetime_str_r(datetime_r=None):
  """
  获取当前时间戳对应的以T作为分割的日期、时间字符串
  :param datetime_r: datetime.datetime对象
  :return: 日期时间微秒字符串 形如 2021.05.20-11.03.21-082161
  """
  if datetime_r is None or not isinstance(datetime_r, datetime.datetime):
    datetime_r = current_datetime_r()
  return datetime_r.strftime("%Y.%m.%d-%H.%M.%S-%f")


@deprecated(new_fn=current_datetime_str_r, version='0.8.0')
def current_datetime_r_str(datetime_r=None):
  return current_datetime_str_r(datetime_r)


def current_date(timestamp=None):
  """
  获取当前日期
  :param timestamp: 时间戳
  :return: 日期时间格式字符串 形如 2018-11-19
  """
  return time.strftime("%Y-%m-%d", time.localtime(timestamp))


def current_date_str(timestamp=None):
  """
  获取当前日期
  :param timestamp: 时间戳
  :return: 日期时间格式字符串 形如 20181119
  """
  return time.strftime("%Y%m%d", time.localtime(timestamp))


def current_time(timestamp=None):
  """
  获取当前时间
  :param timestamp: 时间戳
  :return: 日期时间格式字符串 形如 10:20:35
  """
  return time.strftime("%H:%M:%S", time.localtime(timestamp))


def current_time_str(timestamp=None):
  """
  获取当前时间
  :param timestamp: 时间戳
  :return: 日期时间格式字符串 形如 10:20:35
  """
  return time.strftime("%H%M%S", time.localtime(timestamp))


def current_time_str_m(datetime_r=None):
  """
  获取当前时间
  :param datetime_r: datetime.datetime对象
  :return: 时间/微秒格式字符串 形如 10.20.35-987
  """
  if datetime_r is None or not isinstance(datetime_r, datetime.datetime):
    datetime_r = current_datetime_r()
  return datetime_r.strftime("%H.%M.%S-%f")


class Timer(object):
  """
  计时器类
  """

  # 无效时间戳标志
  _invalid_ts = -1

  def __init__(self, debug=False):
    self._debug = debug
    self._start_ts = self._invalid_ts
    self._finish_ts = self._invalid_ts
    self.reset()

  def reset(self):
    """
    重新计时
    """
    self._start_ts = time.time()
    self._finish_ts = self._invalid_ts
    if self._debug:
      self.display_start('start at: ')

  @property
  def start_ts(self):
    return self._start_ts

  @property
  def finish_ts(self):
    return self._finish_ts

  @property
  def elapsed(self):
    return time.time() - self.start_ts

  def start(self):
    self.reset()
    return self.start_ts

  def finish(self):
    self._finish_ts = time.time()
    return self.finish_ts

  @property
  def stopped(self):
    return self._finish_ts != self._invalid_ts

  def cost(self):
    self._set_finish()
    return self._finish_ts - self._start_ts

  def state_dict(self):
    return {
      'start_ts': self.start_ts,
      'finish_ts': self.finish_ts
    }

  def load_state_dict_(self, state_dict):
    self._start_ts = state_dict['start_ts']
    self._finish_ts = state_dict['finish_ts']

  def display_start(self, msg=None):
    Timer._display_datetime(self._start_ts, msg)

  def display_finish(self, msg=None):
    self._set_finish()
    Timer._display_datetime(self._finish_ts, msg)

  def display_cost(self, msg=None):
    cost = self.cost()
    prefix = ''
    if msg:
      prefix = '{}'.format(msg)
    it_print('{} cost {}'.format(prefix, self.format(cost)))

  @classmethod
  def format(cls, seconds):
    assert seconds >= 0
    seconds = int(seconds)

    if seconds < 60:
      return '{} seconds'.format(seconds)
    elif 60 <= seconds < 120:
      return cls._format('{} minute {}', seconds, 60)
    elif 120 <= seconds < 3600:
      return cls._format('{} minutes {}', seconds, 60)
    elif 3600 <= seconds < 7200:
      return cls._format('{} hour {}', seconds, 3600)
    elif 7200 <= seconds < 86400:
      return cls._format('{} hours {}', seconds, 3600)
    elif 86400 <= seconds < 172800:
      return cls._format('{} day {}', seconds, 86400)
    else:
      return cls._format('{} days {}', seconds, 86400)

  def _set_finish(self):
    if not self.stopped:
      self.finish()

  @staticmethod
  def _display_datetime(ts, msg=None):
    _tmp_msg = current_datetime(ts)
    if msg is not None:
      if not msg.endswith(' '):
        msg += ' '
      _tmp_msg = msg + _tmp_msg
    it_print(_tmp_msg)

  @classmethod
  def _format(cls, fmt, seconds, factor):
    return fmt.format(seconds // factor, cls.format(seconds % factor))

  def __repr__(self):
    msg = ['start at {}'.format(current_datetime(self.start_ts))]
    if self.stopped:
      msg.append('finished at {}'.format(
        current_datetime(self.finish_ts)
      ))
    return ', '.join(msg)


def timed(description=None, num_pre_blanklines=0, num_suf_blanklines=0):
  """
  计时标记
  :param description: 计时描述
  :param num_pre_blanklines: 输出计时前空行数
  :param num_suf_blanklines: 输出计时后空行数
  :return: 包装函数
  """

  def function_wrapper(fn):
    """
        标记为统计运行时间的函数
        :param fn: 被测函数
        """
    assert callable(fn)

    def wrapper(*args, **kwargs):
      _description = fn.__name__
      if description is not None and not callable(description):
        _description = description

      if num_pre_blanklines > 0:
        [it_print() for _ in range(num_pre_blanklines)]
      timer = Timer()
      timer.display_start('{} start at'.format(_description))
      result = fn(*args, **kwargs)
      timer.display_finish('{} complete at'.format(_description))
      timer.display_cost(_description)
      if num_suf_blanklines > 0:
        [it_print() for _ in range(num_pre_blanklines)]

      return result

    return wrapper

  if callable(description):
    return function_wrapper(fn=description)

  return function_wrapper



# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/11/14 16:30
import codecs
import subprocess
import warnings

from .compat import python2
from .utility import current_datetime_r_str
from .utility import write_file_contents

NOHUP = 'nohup'
BG_MARK = '&'


def _get_log_paths(log_prefix):
  time_suffix = current_datetime_r_str()

  def refine_path(prefix, suffix):
    return '{}-{}.{}'.format(prefix, time_suffix, suffix)

  cmd_path = refine_path(log_prefix, 'cmd')
  out_path = refine_path(log_prefix, 'out')
  err_path = refine_path(log_prefix, 'err')

  return cmd_path, out_path, err_path


def shell_invoke(command, **kwargs):
  """
  Shell命令调用
  :param command: 命令
  :param kwargs: 关键字参数

  Args:
      log_prefix: 日志路径前缀
      debug: 是否为调试模式
      daemon: 是否在主线程退出之后仍然运行
      on_error: 用于处理出现错误时的函数
          接收参数 e subprocess.CalledProcessError
      ret_buffer: 返回值缓冲, 用于存储如日志等路径信息
      build_log_paths_fn: 构建日志路径函数, 接受日志前缀，返回cmd_path, out_path, err_path
  :return: 返回码
  """
  log_prefix = kwargs.pop('log_prefix', None)
  debug = kwargs.pop('debug', True)
  daemon = kwargs.pop('daemon', False)
  on_error = kwargs.pop('on_error', None)
  ret_buffer = kwargs.pop('ret_buffer', None)
  if on_error is not None and not callable(on_error):
    message = 'on_error must be of callable, but got {}'
    raise TypeError(message.format(type(on_error)))

  command = command.strip()
  if command == '' or command.startswith(NOHUP) or command.endswith(BG_MARK):
    s = (
      'command should be a string not start with nohup '
      'and not end with &, but got "{}"'
    )
    raise ValueError(s.format(command))

  stdout, stderr = None, None
  if log_prefix is not None:
    build_log_paths_fn = kwargs.pop('build_log_paths_fn', _get_log_paths)
    cmd_path, out_path, err_path = build_log_paths_fn(log_prefix)
    if ret_buffer is not None and isinstance(ret_buffer, dict):
      ret_buffer['cmd_path'] = cmd_path
      ret_buffer['out_path'] = out_path
      ret_buffer['err_path'] = err_path
    write_file_contents(cmd_path, command)
    stdout = codecs.open(out_path, mode='w', encoding='utf-8')
    if not debug:
      stderr = codecs.open(err_path, mode='w', encoding='utf-8')
    else:
      stderr = stdout

  if daemon:
    command = '{} {} {}'.format(NOHUP, command, BG_MARK)

  try:
    return subprocess.check_call(
      command, shell=True, stdout=stdout, stderr=stderr
    )
  except subprocess.CalledProcessError as e:
    if on_error:
      on_error(e)
    else:
      raise


def executable_exists(executable):
  """
  检测可执行文件是否存在
  :param executable: 可执行文件
  :return: 检测结果, 若存在，返回True, 否则返回False
  """
  try:
    cmd = 'which {} >/dev/null'.format(executable)
    subprocess.check_call(cmd, shell=True)
  except subprocess.CalledProcessError:
    return False
  return True


def _python_invoke(command, **kwargs):
  """
  Python命令调用
  :param command: 命令
  :param kwargs: 关键字参数

  Args:
      version: Python版本号, 默认为3
      log_prefix: 日志路径前缀
      debug: 是否为调试模式
      daemon: 是否在主线程退出之后仍然运行
      on_error: 错误处理函数
      interpreter: Python解释器
  :return: 返回码
  """
  command = command.strip()
  if command == '' or command.startswith('python'):
    message = 'python command should be a string not start with python'
    raise ValueError(message)

  interpreter = kwargs.pop('interpreter', None)
  if interpreter is None:
    version = kwargs.pop('version', 3)
    if not (version == 2 or version == 3):
      raise ValueError('only support python2 and python3')
    interpreter = 'python{}'.format(version)
    if not executable_exists(interpreter):
      alternative = 'python'
      params = {
        'interpreter': interpreter,
        'alternative': alternative
      }
      message = (
        '{interpreter} not exists, so use {alternative} as '
        'interpreter, ensure that {alternative} is actually '
        'the same as {interpreter}'
      )
      warnings.warn(message.format(**params))
      interpreter = 'python'

  command = '{} -u {}'.format(interpreter, command)
  return shell_invoke(command, **kwargs)


def python2_invoke(command, **kwargs):
  kwargs['version'] = 2
  return _python_invoke(command, **kwargs)


def python3_invoke(command, **kwargs):
  kwargs['version'] = 3
  return _python_invoke(command, **kwargs)


python_invoke = python2_invoke if python2 else python3_invoke



from .collection import unique_list
from .datetime import Timer
from .datetime import current_date
from .datetime import current_date_str
from .datetime import current_datetime
from .datetime import current_datetime_r
from .datetime import current_datetime_r_str
from .datetime import current_datetime_str
from .datetime import current_datetime_str_d
from .datetime import current_datetime_str_r
from .datetime import current_datetime_str_s
from .datetime import current_time
from .datetime import current_time_str
from .datetime import current_time_str_m
from .datetime import current_timestamp
from .datetime import timed
from .exception import InternalError
from .exception import exit_print
from .exception import raise_print
from .filesystem import absdir
from .filesystem import abspath
from .filesystem import basename
from .filesystem import concat_path
from .filesystem import create_if_not_exists
from .filesystem import dirname
from .filesystem import file_append
from .filesystem import file_exists
from .filesystem import file_lines
from .filesystem import file_md5sum
from .filesystem import file_size
from .filesystem import is_abspath
from .filesystem import load_file_contents
from .filesystem import load_json_contents
from .filesystem import mkdtemp
from .filesystem import open_files
from .filesystem import refine_path
from .filesystem import remove_file
from .filesystem import work_dir
from .filesystem import write_file_contents
from .filesystem import write_iterable_contents
from .filesystem import write_json_contents
from .filesystem import write_line
from .filesystem import write_pid
from .httpclient import HttpTest
from .httpclient import json_print
from .httpclient import print_dict
from .httpclient import print_indent
from .httpclient import print_json
from .httpclient import print_prefix
from .printext import disable_print
from .printext import enable_print
from .printext import is_print_enable
from .printext import it_print
from .printext import it_prints
from .printext import set_print_disable
from .printext import set_print_enable
from .runtime import kill_process
from .stringext import as_json
from .stringext import from_json
from .stringext import md5sum
from .stringext import parse_json
from .stringext import to_json
from .validation import contain_keys
from .validation import in_range
from .webext import TimeoutError
from .webext import build_url
from .webext import fetch_available_port
from .webext import request



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: 王树根
# email: wangshugen@ict.ac.cn
# date: 2018/8/23 14:29
import ssl

from setuptools import find_packages
from setuptools import setup

ssl._create_default_https_context = ssl._create_unverified_context

setup(
  name='easy_tornado',
  version='0.7.1',
  description='A tornado based web framework and toolkit package',
  author='Wang Shugen',
  author_email='wsg1107556314@163.com',
  url='https://artifacts.wshugen.cn/python/easy_tornado',
  packages=find_packages(),
  install_requires=['tornado', 'decorator', 'six', 'urllib3', 'psutil']
)
